#include "DXUT.h"
#include "AnimatedCharacterInstance.h"
#include "AnimatedCharacter.h"
#include "CharacterAnimation.h"
#include "nvutmesh.h"
#include "ArmyManager.h"

AnimatedCharacterInstance::AnimatedCharacterInstance()
{
    iState = IDLE1;
    targetPosition = D3DXVECTOR3(0,0,0);
    m_ReferenceCharacter = NULL;
    color = D3DXVECTOR3(1,1,1);
    lod1 = 1.f;
    lod2 = 2.f;
    lod3 = 3.f;
    animationPlaybackRate = 1.f;
    bLockLOD = false;
}

AnimatedCharacterInstance::~AnimatedCharacterInstance()
{

}

void AnimatedCharacterInstance::Initialize(D3DXVECTOR3 position,D3DXVECTOR3 rotation,D3DXVECTOR3 scale)
{
    assert(m_ReferenceCharacter);
    mPosition = position;
    mRotation = rotation;
    mScale = scale;
    animation = iState = IDLE1;
    this->animationTime = m_ReferenceCharacter->m_animations.size()?m_ReferenceCharacter->m_animations[animation]->duration * ((float)rand()/32768.0f):0;
    bOnlyWorldMatrix = false;
}

void AnimatedCharacterInstance::Initialize(D3DXMATRIX worldMatrix)
{
    mWorld = worldMatrix;
    bOnlyWorldMatrix = true;
    animation = iState = IDLE1;
    this->animationTime = m_ReferenceCharacter->m_animations.size()?m_ReferenceCharacter->m_animations[animation]->duration * ((float)rand()/32768.0f):0;
}

void AnimatedCharacterInstance::RandomMeshAndAnimation(AnimatedCharacter *m_ReferenceCharacter)
{
    this->m_ReferenceCharacter = m_ReferenceCharacter;

    RandomMeshSet(m_ReferenceCharacter);
    RandomAnimation(m_ReferenceCharacter,true);
}

static const D3DXVECTOR3 colors[] = 
{
    D3DXVECTOR3(1.f,1.f,1.f),
    D3DXVECTOR3(0.9f,0.9f,1.f),
    D3DXVECTOR3(1.f,0.9f,1.f),
    D3DXVECTOR3(1.f,0.9f,0.9f),
    D3DXVECTOR3(0.9f,0.9f,1.f),
    D3DXVECTOR3(0.9f,1.f,1.f),
    D3DXVECTOR3(0.9f,1.f,0.9f),
};

static const D3DXVECTOR3 lodcolors[] = 
{
    D3DXVECTOR3(1.f,0.f,0),
    D3DXVECTOR3(0.95f,0.95f,0.2f),
    D3DXVECTOR3(0.4f,0.95f,0.4f),
    D3DXVECTOR3(0.7f,0.f,0.7f),
};

void AnimatedCharacterInstance::RandomMeshSet(AnimatedCharacter *m_ReferenceCharacter)
{
    this->m_ReferenceCharacter = m_ReferenceCharacter;

    int heads[] = {19,20,12};
    int pads[] = {-1,3,5,4};
    int pads2[] = {-1,10,11,9};
    int shields[] = {-1,6,8,7};
    int weapons[] = {0,2,1};
    int torsos[] = {18,16,14};
    int legs[] = {17,15,13};

    meshSet = 0;

    int index = rand()%(sizeof(heads)/sizeof(int));
    index = heads[index];
    if(index != -1)
        meshSet |= 1<<index;

    index = rand()%(sizeof(pads)/sizeof(int));
    index = pads[index];
    if(index != -1)
        meshSet |= 1<<index;

    index = rand()%(sizeof(pads2)/sizeof(int));
    index = pads2[index];
    if(index != -1)
        meshSet |= 1<<index;

    index = rand()%(sizeof(shields)/sizeof(int));
    index = shields[index];
    if(index != -1)
        meshSet |= 1<<index;

    index = rand()%(sizeof(weapons)/sizeof(int));
    index = weapons[index];
    if(index != -1)
        meshSet |= 1<<index;

    index = rand()%(sizeof(torsos)/sizeof(int));
    index = torsos[index];
    if(index != -1)
        meshSet |= 1<<index;

    index = rand()%(sizeof(legs)/sizeof(int));
    index = legs[index];
    if(index != -1)
        meshSet |= 1<<index;

    color = colors[rand() % (sizeof(colors)/sizeof(D3DXVECTOR3))];
    baseColor = color;
}

void AnimatedCharacterInstance::RandomScale(float baseUniform)
{
    mScale.x *= baseUniform*(0.9f + 0.2f*(rand() / 32769.f));
    mScale.y *= baseUniform*(0.8f + 0.4f*(rand() / 32769.f));
    mScale.z *= baseUniform*(0.9f + 0.2f*(rand() / 32769.f));

    if(bOnlyWorldMatrix)
    {
        D3DXMATRIX scale;
        D3DXMatrixScaling(&scale,baseUniform,baseUniform,baseUniform);
        mWorld = scale * mWorld;
    }
}

void AnimatedCharacterInstance::RandomAnimation(AnimatedCharacter *m_ReferenceCharacter,bool bRandomStart)
{
    this->m_ReferenceCharacter = m_ReferenceCharacter;

    iState = (AIState)(rand() % MAX_STATES);
    if(iState == DEAD1) iState = CHEER1;
    else if(iState == DEAD2) iState = CHEER2;
    else if(iState == DEAD3) iState = CHEER3;

    animationPlaybackRate = 0.9f+0.1f*((float)rand()/32768.0f);

    if(m_ReferenceCharacter && m_ReferenceCharacter->m_animations.size() > 0)
    {
        animation = GetAnimationForState(iState);

        if(bRandomStart)
            animationTime = m_ReferenceCharacter->m_animations[animation]->duration * ((float)rand()/32768.0f);
        else
            animationTime = 0.f;
    }
    else
    {
        animation = 0;
        animationTime = 0.f;
    }
}

void AnimatedCharacterInstance::SetState(AIState state)
{
    animationPlaybackRate = 1.f;
    iState = state;
    animation = GetAnimationForState(iState);
}

void AnimatedCharacterInstance::UpdateWorldMatrix()
{
    if(bOnlyWorldMatrix) return;

    D3DXMATRIX translation;
    D3DXMATRIX rotation;
    D3DXMATRIX scale;

    D3DXMatrixTranslation(&translation,mPosition.x,mPosition.y,mPosition.z);
    D3DXMatrixRotationYawPitchRoll(&rotation,mRotation.x,mRotation.y,mRotation.z);
    D3DXMatrixScaling(&scale,mScale.x,mScale.y,mScale.z);
    mWorld = scale * rotation * translation;
}

/*
    UpdateLOD chooses the LOD level to draw this instance at.  It also optionally selects an LOd color based on UI selections.
*/
void AnimatedCharacterInstance::UpdateLOD(const D3DXVECTOR3 &cameraAt,const D3DXVECTOR3 &camDir)
{
    D3DXVECTOR3 current = D3DXVECTOR3(mWorld._41,mWorld._42,mWorld._43);
    current = current - cameraAt;

    D3DXVECTOR3 currentDir;
    D3DXVec3Normalize(&currentDir,&current);

    float distSq = D3DXVec3LengthSq(&current);

    // Simple view culling calculation, but never cull when close(half lod0 radius)
    if(D3DXVec3Dot(&currentDir,&camDir) < 0.7f && distSq >(lod1*lod1)/4.f)
    {
        LOD = -1;
        return;
    }

    if(distSq < lod1*lod1)
    {
        LOD = 0;
    }
    else if(distSq < lod2*lod2)
    {
        LOD = 1;
    }
    else if(distSq < lod3*lod3)
    {
        LOD = 2;
    }
    else
    {
        LOD = 3;
    }

    // $ For debug to show where the LOD line is.
    if(ArmyManager::singleton()->GetVisualizeLOD())
        color = lodcolors[LOD];
    else
        color = baseColor;
}

float AnimatedCharacterInstance::GetRandomYaw()
{
    return 360.f * ((float)rand()/32768.0f);
}

/*
    this is a hardcoded list of animations that we expect to be in the mesh file.  
    
    If it can't find the right animation for the state it is in, returns the first animation in the list.
*/
int AnimatedCharacterInstance::GetAnimationForState(AIState state)
{
    static const char *animationNames[] = 
    {
        "Dead3",
        "Dead2",
        "Dead1",
        "Idle2",
        "JumpCheer",
        "Idle1",
        "Cheer3",
        "Cheer2",
        "Cheer1"
    };

    for(int i=0;i<(int)m_ReferenceCharacter->m_animations.size();i++)
    {
        if(strcmp(animationNames[(int)state],m_ReferenceCharacter->m_animations[i]->name.c_str()) == 0)
        {
            return i;
        }
    }
    return 0;
}

/*
    just loop the animation we've chosen.
*/
void AnimatedCharacterInstance::Update(float deltatime, const D3DXVECTOR3 &cameraAt,const D3DXVECTOR3 &camDir)
{
    animationTime += animationPlaybackRate*deltatime;
    
    // only do AI if we have a base character to refer to
    if(m_ReferenceCharacter && m_ReferenceCharacter->m_animations.size() && animationTime > m_ReferenceCharacter->m_animations[animation]->duration) 
    {
            animationTime = 0.f;
    }

    UpdateWorldMatrix();
    UpdateLOD(cameraAt,camDir);
}
