#pragma once 

class AnimatedCharacter;

/*
    A placed and animating character instance.  Simply wraps up the position, animation, and mesh set for this instance.
*/
class AnimatedCharacterInstance
{
public:

    // Character logic
    enum AIState
    {
        DEAD3 = 0,
        DEAD2 = 1,
        DEAD1 = 2,
        IDLE2 = 3,
        JUMPCHEER = 4,
        IDLE1 = 5,
        CHEER3 = 6,
        CHEER2 = 7,
        CHEER1 = 8,
        MAX_STATES = 9
    };

    AnimatedCharacterInstance();
    ~AnimatedCharacterInstance();

    void Initialize(D3DXVECTOR3 position,D3DXVECTOR3 rotation,D3DXVECTOR3 scale=D3DXVECTOR3(1,1,1));
    void Initialize(D3DXMATRIX worldMatrix);
    void SetLODParams(float lod1, float lod2, float lod3){if(bLockLOD) return; this->lod1 = lod1;this->lod2 = lod2;this->lod3 = lod3;};
    void LockLODParams(bool bLock=true){bLockLOD = true;}
    
    void RandomMeshAndAnimation(AnimatedCharacter *m_ReferenceCharacter);
    void RandomAnimation(AnimatedCharacter *m_ReferenceCharacter,bool bRandomStart=true);
    void RandomMeshSet(AnimatedCharacter *m_ReferenceCharacter);
    void RandomScale(float baseUniform=1.f);

    void SetState(AIState state);

    float GetRandomYaw();
    
    void UpdateWorldMatrix();
    void UpdateLOD(const D3DXVECTOR3 &cameraAt,const D3DXVECTOR3 &camDir);
    void Update(float deltatime, const D3DXVECTOR3 &cameraAt,const D3DXVECTOR3 &camDir);

    AnimatedCharacter *m_ReferenceCharacter;
    int animation;
    float animationTime;
    float animationPlaybackRate;
    int meshSet;
    int LOD;    // 0 is full, goes down from there
    D3DXVECTOR3 color;
    D3DXVECTOR3 baseColor;
    D3DXMATRIX mWorld;
    bool bLockLOD;


protected:

    bool bOnlyWorldMatrix;
    D3DXVECTOR3 mPosition;
    D3DXVECTOR3 mRotation;
    D3DXVECTOR3 mScale;

    float lod1;
    float lod2;
    float lod3;

    int GetAnimationForState(AIState state);

    static char **animationNames;

    AIState        iState;
    D3DXVECTOR3 targetPosition;
};

