#include "DXUT.h"
#include "SDKmisc.h"
#include "DXUTcamera.h"
#include "sdkmesh_old.h"
#include "nvutmesh.h"

#include <vector>

#include "ArmyManager.h"
#include "TextureLibray.h"
#include "CharacterAnimation.h"
#include "AnimatedCharacterInstance.h"
#include "DX9AnimationLoader.h"

ArmyManager*ArmyManager::m_Singleton = NULL;

ArmyManager::ArmyManager()
{
    m_pEffect10 = NULL;
    m_pTechAnimation[0] = NULL;
    m_pTechAnimation[1] = NULL;
    m_pTechAnimation[2] = NULL;
    m_pTechAnimationInstanced[0] = NULL;
    m_pTechAnimationInstanced[1] = NULL;
    m_pTechAnimationInstanced[2] = NULL;
    
    m_CharacterLODs[0] = NULL;
    m_CharacterLODs[1] = NULL;
    m_CharacterLODs[2] = NULL;
    m_CharacterLODs[3] = NULL;

    m_pAnimationTexture[0] = NULL;
    m_pAnimationTexture[1] = NULL;
    m_pAnimationTexture[2] = NULL;
    m_pAnimationTexture[3] = NULL;

    m_pAnimationResourceView[0] = NULL;
    m_pAnimationResourceView[1] = NULL;
    m_pAnimationResourceView[2] = NULL;
    m_pAnimationResourceView[3] = NULL;

    m_pWorldViewProjectionVar = NULL;
    m_MaxCharacters = 0;
    m_NumToDraw = 0;
    m_iInstancingMode = 1;
    m_ReferenceCharacter = NULL;
    m_pInputLayout = NULL;
    m_pInputLayoutTerrain = NULL;
    m_pDataTemp = NULL;
    m_bVisualizeLOD = false;
    
    m_pHDRRT = NULL;
    m_pHDRRTSRV = NULL;
    m_pHDRRTRTV = NULL;
    m_pHDRRT2 = NULL;
    m_pHDRRT2SRV = NULL;
    m_pHDRRT2RTV = NULL;
    m_pHDRRT3 = NULL;
    m_pHDRRT3SRV = NULL;
    m_pHDRRT3RTV = NULL;
    m_pSMRT = NULL;
    m_pSMRTSRV = NULL;
    m_pSMRTRTV = NULL;
    
    m_pHDRDepth = NULL;
    m_pHDRDepthDSV = NULL;
    m_bHDR = true;
    m_bShadows = true;
    m_bDisableUpdates=false;

}

/*
    Make the Texture resource, RTV, and SRV suitable for a render target.
*/
void CreateWithViews(ID3D10Device *pd3dDevice,UINT w, UINT h,ID3D10Texture2D **ppTex, ID3D10ShaderResourceView **ppSRV, ID3D10RenderTargetView **ppRTV,DXGI_FORMAT format=DXGI_FORMAT_R10G10B10A2_UNORM)
{
    HRESULT hr;
    D3D10_TEXTURE2D_DESC desc;
    ZeroMemory( &desc, sizeof(D3D10_TEXTURE2D_DESC) );
    desc.ArraySize = 1;
    desc.BindFlags = D3D10_BIND_RENDER_TARGET | D3D10_BIND_SHADER_RESOURCE;
    desc.Format = format;
    desc.Width = w;
    desc.Height = h;
    desc.Usage = D3D10_USAGE_DEFAULT;
    desc.SampleDesc.Count = 1;
    desc.MipLevels = 1;
    V(pd3dDevice->CreateTexture2D(&desc,NULL,ppTex));

    D3D10_RENDER_TARGET_VIEW_DESC RTVDesc;
    ZeroMemory( &RTVDesc, sizeof(RTVDesc) );
    RTVDesc.ViewDimension = D3D10_RTV_DIMENSION_TEXTURE2D;
    RTVDesc.Texture2D.MipSlice = 0;
    RTVDesc.Format = desc.Format;
    V(pd3dDevice->CreateRenderTargetView(*ppTex,&RTVDesc,ppRTV));

    D3D10_SHADER_RESOURCE_VIEW_DESC SRVDesc;
    ZeroMemory( &SRVDesc, sizeof(SRVDesc) );
    SRVDesc.Format = desc.Format;
    SRVDesc.ViewDimension = D3D10_SRV_DIMENSION_TEXTURE2D;
    SRVDesc.Texture2D.MipLevels = desc.MipLevels;
    V(pd3dDevice->CreateShaderResourceView( *ppTex, &SRVDesc, ppSRV ));
}

HRESULT ArmyManager::Initialize(ID3D10Device* pd3dDevice, int bbWidth, int bbHeight)
{
    HRESULT hr;
    WCHAR str[MAX_PATH];

    m_pDataTemp = new InstanceDataElement[INTERNAL_MAX_INSTANCES_PER_BUFFER];

    const D3D10_INPUT_ELEMENT_DESC meshonlylayout[] =
    {
        // Normal character mesh data, setup for GPU skinning
        { "POSITION",    0, DXGI_FORMAT_R32G32B32_FLOAT,        0, 0,  D3D10_INPUT_PER_VERTEX_DATA, 0 },
        { "NORMAL",      0, DXGI_FORMAT_R32G32B32_FLOAT,        0, 12, D3D10_INPUT_PER_VERTEX_DATA, 0 },
        { "TEXCOORD",    0, DXGI_FORMAT_R32G32_FLOAT,           0, 24, D3D10_INPUT_PER_VERTEX_DATA, 0 },
        { "TANGENT",     0, DXGI_FORMAT_R32G32B32_FLOAT,        0, 32, D3D10_INPUT_PER_VERTEX_DATA, 0 },
        { "BONES",       0, DXGI_FORMAT_R32G32B32A32_FLOAT,     0, 44, D3D10_INPUT_PER_VERTEX_DATA, 0 },
        { "WEIGHTS",     0, DXGI_FORMAT_R32G32B32A32_FLOAT,     0, 60, D3D10_INPUT_PER_VERTEX_DATA, 0 },    
    };

    // First load character meshes (load compressed as it is faster)

    DX9AnimationLoader loader;
    V_RETURN(loader.load(pd3dDevice,L"Dwarves\\dwarf-lod0.X.gz",meshonlylayout,sizeof(meshonlylayout)/sizeof(D3D10_INPUT_ELEMENT_DESC),true));
    V(loader.pAnimatedCharacter->Initialize(pd3dDevice));
    m_CharacterLODs[0] = loader.pAnimatedCharacter;

    m_ReferenceCharacter = m_CharacterLODs[0];

    V_RETURN(loader.load(pd3dDevice,L"Dwarves\\dwarf-lod1.X.gz",meshonlylayout,sizeof(meshonlylayout)/sizeof(D3D10_INPUT_ELEMENT_DESC),true));
    V(loader.pAnimatedCharacter->Initialize(pd3dDevice));
    m_CharacterLODs[1] = loader.pAnimatedCharacter;

    V_RETURN(loader.load(pd3dDevice,L"Dwarves\\dwarf-lod2.X.gz",meshonlylayout,sizeof(meshonlylayout)/sizeof(D3D10_INPUT_ELEMENT_DESC),true));
    V(loader.pAnimatedCharacter->Initialize(pd3dDevice));
    m_CharacterLODs[2] = loader.pAnimatedCharacter;

    V_RETURN(loader.load(pd3dDevice,L"Dwarves\\dwarf-lod3.X.gz",meshonlylayout,sizeof(meshonlylayout)/sizeof(D3D10_INPUT_ELEMENT_DESC),true));
    V(loader.pAnimatedCharacter->Initialize(pd3dDevice));
    m_CharacterLODs[3] = loader.pAnimatedCharacter;

    // the matrix palette is only needed for non instanced
    LPCSTR sNumBones = new CHAR[MAX_PATH];
    sprintf_s((char*)sNumBones,MAX_PATH,"%d",max(1,m_ReferenceCharacter->numBones));
    LPCSTR sMaxInstanceConsts = new CHAR[MAX_PATH];
    sprintf_s((char*)sMaxInstanceConsts,MAX_PATH,"%d",INTERNAL_MAX_INSTANCES_PER_BUFFER);

    D3D10_SHADER_MACRO mac[3] =
    {
        { "MATRIX_PALETTE_SIZE_DEFAULT", sNumBones },
        { "MAX_INSTANCE_CONSTANTS", sMaxInstanceConsts},
        { NULL,                          NULL }
    };

    // Read the D3DX effect file
    V_RETURN( DXUTFindDXSDKMediaFileCch( str, MAX_PATH, L"SkinnedInstancing.fx" ) );
    V_RETURN( D3DX10CreateEffectFromFile( str, mac, NULL, "fx_4_0", D3D10_SHADER_ENABLE_STRICTNESS, 0, pd3dDevice, NULL, NULL, &m_pEffect10, NULL, &hr ) );

    delete [] sNumBones;
    delete [] sMaxInstanceConsts;

    // Grab some pointers to techniques
    m_pTechAnimationInstanced[0] = m_pEffect10->GetTechniqueByName( "RenderAnimationInstanced" );
    m_pTechAnimationInstanced[1] = m_pEffect10->GetTechniqueByName( "RenderAnimationInstanced_MidLOD" );
    m_pTechAnimationInstanced[2] = m_pEffect10->GetTechniqueByName( "RenderAnimationInstanced_FarLOD" );

    m_pTechAnimation[0] = m_pEffect10->GetTechniqueByName( "RenderAnimation" );
    m_pTechAnimation[1] = m_pEffect10->GetTechniqueByName( "RenderAnimation_MidLOD" );
    m_pTechAnimation[2] = m_pEffect10->GetTechniqueByName( "RenderAnimation_FarLOD" );

    m_pTechTerrain = m_pEffect10->GetTechniqueByName( "RenderTerrain" );
    m_pTechAnimInstAO = m_pEffect10->GetTechniqueByName( "RenderSceneWithAnimationInstanced_AO" );
    m_pTechHDR = m_pEffect10->GetTechniqueByName( "RenderSceneHDRtoLDR" );
    
    // Grab some pointers to effects variables
    m_pWorldViewProjectionVar = m_pEffect10->GetVariableBySemantic( "WorldViewProjection" )->AsMatrix();
    m_pWorldITVar = m_pEffect10->GetVariableBySemantic( "WorldIT" )->AsMatrix();
    m_pWorldVar = m_pEffect10->GetVariableBySemantic( "World" )->AsMatrix();
    m_pViewProjVar = m_pEffect10->GetVariableBySemantic( "ViewProjection" )->AsMatrix();
    m_pViewportSizeVar = m_pEffect10->GetVariableBySemantic( "ViewportSizePixels" )->AsVector();

    m_pAnimationsVar = m_pEffect10->GetVariableBySemantic( "ANIMATIONS" )->AsShaderResource();
    m_pDiffuseTexVar = m_pEffect10->GetVariableBySemantic( "DIFFUSE" )->AsShaderResource();
    m_pNormalTexVar = m_pEffect10->GetVariableBySemantic( "NORMAL" )->AsShaderResource();
    m_pEnvMapTexVar = m_pEffect10->GetVariableBySemantic( "ENVMAP" )->AsShaderResource();

    // Create our vertex input layouts
    D3D10_PASS_DESC PassDesc;
    V_RETURN( m_pTechAnimation[0]->GetPassByIndex( 0 )->GetDesc( &PassDesc ) );
    V_RETURN( pd3dDevice->CreateInputLayout( meshonlylayout, sizeof(meshonlylayout)/sizeof(D3D10_INPUT_ELEMENT_DESC), PassDesc.pIAInputSignature, PassDesc.IAInputSignatureSize, &m_pInputLayout ) );

    // Initialize our numbers with the passed in value
    m_MaxCharacters = GetMaxCharacters(); 
    SetNumCharacters(m_MaxCharacters); // set to max whichwill initialize all potential characters...

    // Create a texture with all m_animations in it
    V(CreateAnimationTexture(pd3dDevice,m_CharacterLODs[0],&m_pAnimationTexture[0],&m_pAnimationResourceView[0]));
    V(CreateAnimationTexture(pd3dDevice,m_CharacterLODs[1],&m_pAnimationTexture[1],&m_pAnimationResourceView[1]));
    V(CreateAnimationTexture(pd3dDevice,m_CharacterLODs[2],&m_pAnimationTexture[2],&m_pAnimationResourceView[2]));
    V(CreateAnimationTexture(pd3dDevice,m_CharacterLODs[3],&m_pAnimationTexture[3],&m_pAnimationResourceView[3]));

    // Make a simple ground plane for m_Characters to walk on.
    const D3D10_INPUT_ELEMENT_DESC groundLayout[] =
    {
        { "POSITION",    0, DXGI_FORMAT_R32G32B32_FLOAT,        0, 0,  D3D10_INPUT_PER_VERTEX_DATA, 0 },
        { "NORMAL",        0, DXGI_FORMAT_R32G32B32_FLOAT,        0, 12, D3D10_INPUT_PER_VERTEX_DATA, 0 },
        { "TEXCOORD",    0, DXGI_FORMAT_R32G32_FLOAT,        0, 24, D3D10_INPUT_PER_VERTEX_DATA, 0 },
        { "TANGENT",     0, DXGI_FORMAT_R32G32B32_FLOAT,        0, 32, D3D10_INPUT_PER_VERTEX_DATA, 0 },
    };

    V_RETURN( m_pTechTerrain->GetPassByIndex( 0 )->GetDesc( &PassDesc ) );
    V_RETURN( pd3dDevice->CreateInputLayout( groundLayout, sizeof(groundLayout)/sizeof(D3D10_INPUT_ELEMENT_DESC), PassDesc.pIAInputSignature, PassDesc.IAInputSignatureSize, &m_pInputLayoutTerrain ) );
    m_GroundMesh.CreateWithNormalMaps(pd3dDevice,L"Arena\\arena-final.x",(D3D10_INPUT_ELEMENT_DESC*)groundLayout,sizeof(groundLayout)/sizeof(D3D10_INPUT_ELEMENT_DESC),false);

    ID3D10Texture2D *pTexture10 = NULL;
    ID3D10ShaderResourceView *pSRV10 = NULL;
    std::string name;

    name = "cubemap";
    if( SUCCEEDED(DXUTFindDXSDKMediaFileCch( str, MAX_PATH, L"SkyCubemap.dds" )) )
    {
        ID3D10Resource *pRes = NULL;
        hr = D3DX10CreateTextureFromFile( pd3dDevice, str, NULL, NULL, &pRes,&hr );
        if( SUCCEEDED(hr) && pRes )
        {
            pRes->QueryInterface( __uuidof( ID3D10Texture2D ), (LPVOID*)&pTexture10 );
            D3D10_TEXTURE2D_DESC desc;
            pTexture10->GetDesc( &desc );
            D3D10_SHADER_RESOURCE_VIEW_DESC SRVDesc;
            ZeroMemory( &SRVDesc, sizeof(SRVDesc) );
            SRVDesc.Format = desc.Format;
            SRVDesc.ViewDimension = D3D10_SRV_DIMENSION_TEXTURECUBE;
            SRVDesc.TextureCube.MipLevels = desc.MipLevels;
            pd3dDevice->CreateShaderResourceView( pTexture10, &SRVDesc, &pSRV10 );
            SAFE_RELEASE( pRes );

            TextureLibrary::singleton()->AddTexture(name.c_str(),pTexture10,pSRV10);
            m_pEnvMapTexVar->SetResource(pSRV10);
            SAFE_RELEASE(pTexture10);
            SAFE_RELEASE(pSRV10);
        }
    }

    // skybox
    m_skybox.OnCreateDevice(pd3dDevice);    
    m_skybox.SetTexture(TextureLibrary::singleton()->GetShaderResourceView("cubemap"));
    ResizedSwapChain(pd3dDevice,bbWidth,bbHeight);

    CreateWithViews(pd3dDevice,1024,2048,&m_pSMRT,&m_pSMRTSRV,&m_pSMRTRTV, DXGI_FORMAT_R32_FLOAT);

    return S_OK;
}

/* 
    Swap chain size dependant resources... 
*/
void ArmyManager::ResizedSwapChain(ID3D10Device *pd3dDevice, int bbWidth, int bbHeight)
{
    HRESULT hr;

    SAFE_RELEASE(m_pHDRRT);
    SAFE_RELEASE(m_pHDRRTRTV);
    SAFE_RELEASE(m_pHDRRTSRV);

    SAFE_RELEASE(m_pHDRRT2);
    SAFE_RELEASE(m_pHDRRT2RTV);
    SAFE_RELEASE(m_pHDRRT2SRV);

    SAFE_RELEASE(m_pHDRRT3);
    SAFE_RELEASE(m_pHDRRT3RTV);
    SAFE_RELEASE(m_pHDRRT3SRV);

    SAFE_RELEASE(m_pHDRDepth);
    SAFE_RELEASE(m_pHDRDepthDSV);

    CreateWithViews(pd3dDevice,bbWidth,bbHeight,&m_pHDRRT,&m_pHDRRTSRV,&m_pHDRRTRTV);
    CreateWithViews(pd3dDevice,bbWidth,bbHeight,&m_pHDRRT2,&m_pHDRRT2SRV,&m_pHDRRT2RTV);
    CreateWithViews(pd3dDevice,bbWidth,bbHeight,&m_pHDRRT3,&m_pHDRRT3SRV,&m_pHDRRT3RTV);
        
    D3D10_TEXTURE2D_DESC desc;
    D3D10_DEPTH_STENCIL_VIEW_DESC DSVDesc;
    ZeroMemory( &desc, sizeof(D3D10_TEXTURE2D_DESC) );
    desc.ArraySize = 1;
    desc.Width = bbWidth;
    desc.Height = bbHeight;
    desc.Usage = D3D10_USAGE_DEFAULT;
    desc.SampleDesc.Count = 1;
    desc.MipLevels = 1;
    desc.Format = DXGI_FORMAT_D32_FLOAT;
    desc.BindFlags = D3D10_BIND_DEPTH_STENCIL;
    V(pd3dDevice->CreateTexture2D(&desc,NULL,&m_pHDRDepth));

    ZeroMemory( &DSVDesc, sizeof(DSVDesc) );
    DSVDesc.ViewDimension = D3D10_DSV_DIMENSION_TEXTURE2D;
    DSVDesc.Format = desc.Format;
    V(pd3dDevice->CreateDepthStencilView(m_pHDRDepth,&DSVDesc,&m_pHDRDepthDSV));

    SetViewportSize(bbWidth,bbHeight);
}

void ArmyManager::SetViewportSize(int w,int h)
{
    if(m_pViewportSizeVar)
    {
        D3DXVECTOR4 v;
        v.x = (float)w;
        v.y = (float)h; v.z = 0;v.w = 0;
        m_pViewportSizeVar->SetFloatVector((float*)&v);
    }
}

/*
    This sets the # of characters drawn, and algorithmically places them around the world.
*/
void ArmyManager::SetNumCharacters(int num)
{
    if(num > m_MaxCharacters) num = m_MaxCharacters;

    // assign the # we want to Draw
    m_NumToDraw = num;

    //  If the character object list is empty then we need to make our characters and initalize positions
    //  NOTE, this is pretty suboptimal design, however to handle the case where the max characters changes this is here
    //      if the max drawable characters changes, then the characters list is emptied and thus remade here.
    if(m_Characters.size() == 0)
    {
        D3DXMATRIX mScale;
        D3DXMATRIX mTranslation;
        D3DXMATRIX mRotation;
        D3DXMATRIX mWorld;
        AnimatedCharacterInstance *character = NULL;

        // a bunch of character instances. sort of fit to the arena... 
        int charactersLeft = m_MaxCharacters - ((int)m_Characters.size());

        // Add some full LOD dead guys and a fighter
        D3DXMATRIX deadGuyPosition;

        D3DXMatrixTranslation(&deadGuyPosition,100.f,4.5f,10.f);
        character = new AnimatedCharacterInstance();
        character->RandomMeshSet(m_ReferenceCharacter);
        character->Initialize(deadGuyPosition);
        character->SetState(AnimatedCharacterInstance::IDLE1);
        character->RandomScale(2.f);
        character->SetLODParams(2000.f,2001.f,2002.f);
        character->LockLODParams();
        m_Characters.push_back(character);
        charactersLeft--;

        D3DXMatrixTranslation(&deadGuyPosition,50.f,4.5f,-25.f);
        character = new AnimatedCharacterInstance();
        character->RandomMeshSet(m_ReferenceCharacter);
        character->Initialize(deadGuyPosition);
        character->SetState(AnimatedCharacterInstance::DEAD1);
        character->RandomScale(5.f);
        character->SetLODParams(2000.f,2001.f,2002.f);
        character->LockLODParams();
        m_Characters.push_back(character);
        charactersLeft--;

        D3DXMatrixTranslation(&deadGuyPosition,150.f,4.5f,20.f);
        character = new AnimatedCharacterInstance();
        character->RandomMeshSet(m_ReferenceCharacter);
        character->Initialize(deadGuyPosition);
        character->SetState(AnimatedCharacterInstance::DEAD2);
        character->RandomScale(5.f);
        character->SetLODParams(2000.f,2001.f,2002.f);
        character->LockLODParams();
        m_Characters.push_back(character);
        charactersLeft--;

        // Now we read the remaining characters from an exported positions file...
        float rowHeight = m_ReferenceCharacter->boundingRadius/1.5f;
        float rowDepth = m_ReferenceCharacter->boundingRadius/1.5f;
        int currentRow = 0;
        int charactersPerRow = 90;
        int startCharacter = 0;
        WCHAR str[MAX_PATH];
        if(SUCCEEDED(DXUTFindDXSDKMediaFileCch( str, MAX_PATH, L"Positions.txt" )) )
        {
            char filename[MAX_PATH];
            WideCharToMultiByte(CP_ACP,0,str,-1,filename,MAX_PATH," ",FALSE);
            FILE * file = NULL;
            fopen_s(&file,filename,"r");
            assert(file);

            for(int iCharacter=0;iCharacter<charactersLeft;iCharacter++)
            {
                D3DXMATRIX inputMatrix;

                if(feof(file)) break;

                // Loading a matrix, but we only use the position for now...
                fscanf_s(file,"(%f,%f,%f,%f,"
                            "%f,%f,%f,%f,"
                            "%f,%f,%f,%f,"
                            "%f,%f,%f,%f)\n",
                            &inputMatrix._11,&inputMatrix._12,&inputMatrix._13,&inputMatrix._14,
                            &inputMatrix._21,&inputMatrix._22,&inputMatrix._23,&inputMatrix._24,
                            &inputMatrix._31,&inputMatrix._32,&inputMatrix._33,&inputMatrix._34,
                            &inputMatrix._41,&inputMatrix._42,&inputMatrix._43,&inputMatrix._44);

                // This just converts the exported data matrices from Max space to our world space.
                D3DXMATRIX MAXtoD3D;
                memset(&MAXtoD3D,0,sizeof(D3DXMATRIX));
                MAXtoD3D._11 = 1;MAXtoD3D._23 = 1;MAXtoD3D._32 = -1;MAXtoD3D._44 = 1;
                D3DXMatrixMultiply(&inputMatrix,&MAXtoD3D,&inputMatrix);
                D3DXMatrixInverse(&MAXtoD3D,NULL,&MAXtoD3D);
                D3DXMatrixMultiply(&inputMatrix,&inputMatrix,&MAXtoD3D);

                D3DXVECTOR4 position = D3DXVECTOR4(0,0,0,1);
                D3DXVec4Transform(&position,&position,&inputMatrix);

                // Rotate each character to face the "center" by orienting them to the closest point on a line along the middle of the arena.
                D3DXVECTOR3 posTemp  = D3DXVECTOR3(position.x,position.y,position.z);
                D3DXVECTOR3 endPointA = D3DXVECTOR3(-150,posTemp.y,0);    // pull height from this objects position...
                D3DXVECTOR3 endPointB = D3DXVECTOR3(370,posTemp.y,0);
                posTemp -= endPointA;   // put our position in relation to the line...
                D3DXVECTOR3 lineDir = endPointB - endPointA;
                float lineLength = D3DXVec3Length(&lineDir);
                D3DXVec3Normalize(&lineDir,&lineDir);
                float transAlongLine = min(lineLength,max(0,D3DXVec3Dot(&posTemp,&lineDir)));
                D3DXVECTOR3 targetPoint = endPointA + transAlongLine*lineDir;

                // 4 for now just use a point
                D3DXVECTOR3 direction = D3DXVECTOR3(position.x,position.y,position.z) - targetPoint;
                D3DXVec3Normalize(&direction,&direction);
                D3DXVECTOR3 up = D3DXVECTOR3(0,1,0);
                D3DXVECTOR3 right;
                D3DXVec3Cross(&right,&up,&direction);

                D3DXMATRIX rotation = D3DXMATRIX(right.x,right.y,right.z,0,up.x,up.y,up.z,0,direction.x,direction.y,direction.z,0,0,0,0,1);

                inputMatrix = rotation * inputMatrix;

                // Make and add in our brand new character
                character = new AnimatedCharacterInstance();
                character->RandomMeshSet(m_ReferenceCharacter);
                character->Initialize(inputMatrix);
                character->RandomScale(1.f);
                character->RandomAnimation(m_ReferenceCharacter,true);
                character->SetLODParams(70.f,150.f,390.f);
                m_Characters.push_back(character);
                
            }
            fclose(file);
        }
        else 
        {
            for(int iCharacter=0;iCharacter<charactersLeft;iCharacter++)
            {
                int iLocalCharacter = iCharacter/2;

                // detect row change and increment to allow widening of rows as we go.
                if(iLocalCharacter > 0 && iLocalCharacter-startCharacter == charactersPerRow) 
                {
                    currentRow++;
                    startCharacter = iLocalCharacter;
                }

                D3DXMATRIX mTranslation;
                D3DXMatrixTranslation(&mTranslation,
                                    m_ReferenceCharacter->boundingRadius*(float)(iLocalCharacter-startCharacter),
                                    rowHeight*(float)currentRow,rowDepth*(float)currentRow);

                D3DXVECTOR3 position = D3DXVECTOR3(-(charactersPerRow/2.f)*m_ReferenceCharacter->boundingRadius,
                                                    41,
                                                    150);
                D3DXMATRIX standRotation;
                switch(iCharacter %2)
                {
                case 1:
                    D3DXMatrixRotationY(&standRotation,0);
                    break;
                case 0:
                    D3DXMatrixRotationY(&standRotation,D3DX_PI);
                    break;
                }

                D3DXVECTOR3 rotation = D3DXVECTOR3(D3DX_PI,0,0);

                D3DXVECTOR4 temp;
                D3DXMATRIX realTranslation = mTranslation * standRotation;
                D3DXVec3Transform(&temp,&position,&realTranslation);
                memcpy(&position,&temp,sizeof(D3DXVECTOR3));

                character = new AnimatedCharacterInstance();
                character->RandomMeshSet(m_ReferenceCharacter);
                character->Initialize(position,rotation);
                character->RandomScale(1.f);
                character->RandomAnimation(m_ReferenceCharacter,true);
                character->SetLODParams(m_ReferenceCharacter->boundingRadius*10.f,
                                        m_ReferenceCharacter->boundingRadius*30.f,
                                        m_ReferenceCharacter->boundingRadius*50.f);
                m_Characters.push_back(character);
            }
        }
    }
}

void ArmyManager::SetLODParams(float lod1, float lod2, float lod3)
{
    for(int i=0;i<(int)m_Characters.size();i++)
    {
        m_Characters[i]->SetLODParams(lod1,lod2,lod3);
    }
}

/*
    This calls to each character instance to Update it's animation data
*/
HRESULT ArmyManager::Update(float deltaTime,const D3DXVECTOR3 &cameraAt, const D3DXVECTOR3 &camDir)
{
    m_pEffect10->GetVariableByName("g_EyePos")->AsVector()->SetFloatVector((float*)&cameraAt);

    for(int i=0;i<(int)m_Characters.size();i++)
    {
        m_Characters[i]->Update(deltaTime,cameraAt,camDir);
    }
    return S_OK;
}

/*
    Update the instance data buffer with new animation data, then Draw
*/
HRESULT ArmyManager::Render(ID3D10Device* pd3dDevice,D3DXMATRIX &mView,D3DXMATRIX &mProj, float time)
{
    HRESULT hr;
    float ClearColor[4] = { 0.4f, 0.8f, 1.f, 0.0f };

    m_polysDrawnLastFrame = 0;
    m_drawCallsLastFrame = 0;

    if(!m_Characters.size()) return S_OK;

    // common to all techniques variables
    D3DXMATRIX viewProj = mView * mProj;
    m_pViewProjVar->SetMatrix((float*)&viewProj);

    ID3D10RenderTargetView *pOldRTV = DXUTGetD3D10RenderTargetView();
    ID3D10DepthStencilView *pOldDSV = DXUTGetD3D10DepthStencilView();

    // Update characters and draw state, updating buffers if instancing...
    if(!m_bDisableUpdates)
        V(UpdateInstancedDrawState(pd3dDevice,m_iInstancingMode>0));

    m_pEffect10->GetVariableByName("g_bShadow")->AsScalar()->SetBool(m_bShadows);
    // We have implemented a very simple orthographic shadow pass
    //      So need to render high lod characters into the shadow buffer
    if(m_bShadows)   // "shadows"
    {
        float ShadowClearColor[4] = { 1.f, 1.0f, 1.f, 0.0f };

        ID3D10ShaderResourceView *pNulls[] = {NULL,NULL,NULL};
        pd3dDevice->PSSetShaderResources(0,3,pNulls);
        pd3dDevice->ClearRenderTargetView( m_pSMRTRTV, ShadowClearColor );
        pd3dDevice->OMSetRenderTargets(1,&m_pSMRTRTV,NULL);
        D3D10_VIEWPORT shadowVP;
        D3D10_VIEWPORT oldVP;
        UINT one = 1;

        pd3dDevice->RSGetViewports(&one,&oldVP);
        shadowVP = oldVP;
        D3D10_TEXTURE2D_DESC smDesc;
        m_pSMRT->GetDesc(&smDesc);
        shadowVP.Width = smDesc.Width;
        shadowVP.Height = smDesc.Height;
        pd3dDevice->RSSetViewports(1,&shadowVP);

        D3DXMATRIX shadowView;
        D3DXMATRIX shadowProj;
        D3DXMatrixOrthoLH(&shadowProj,1000.f,2000.f,0.f,600.f);
        D3DXVECTOR3 lightPos = D3DXVECTOR3(0.f,500.f,0.f);
        D3DXVECTOR3 lightAt = D3DXVECTOR3(0,0,0);
        D3DXVECTOR3 up = D3DXVECTOR3(1,0,0);
        D3DXMatrixLookAtLH(&shadowView,&lightPos,&lightAt,&up);

        D3DXMATRIX mVP = shadowView * shadowProj;

        m_pEffect10->GetVariableBySemantic("SHADOWTRANSFORM")->AsMatrix()->SetMatrix((float*)&mVP);

        pd3dDevice->IASetPrimitiveTopology(D3D10_PRIMITIVE_TOPOLOGY_TRIANGLELIST);
        switch(m_iInstancingMode)
        {
            /// Single Draws
        case 0:
            V(DrawSorted(pd3dDevice,shadowView,shadowProj,time,0));
            break;
            // Hybrid instancing
        case 1:
        default:
            V(DrawHybridInstanced(pd3dDevice,shadowView,shadowProj,time,0));
            break;
        }

        pd3dDevice->RSSetViewports(1,&oldVP);

        m_pEffect10->GetVariableBySemantic("SHADOWMAP")->AsShaderResource()->SetResource(m_pSMRTSRV);

        //D3DX10SaveTextureToFile(m_pSMRT,D3DX10_IFF_DDS,L"SMOutput.dds");
        pd3dDevice->OMSetRenderTargets(1,&pOldRTV,pOldDSV);
    }

    // If enabled we render the scene to a higher precision buffer which allows the color value to be higher than 1
    //  And then we have a gamma and contract adjust pass at the end.
    if(m_bHDR)
    {
        // Set HDr Target
        pd3dDevice->ClearRenderTargetView( m_pHDRRTRTV, ClearColor );
        pd3dDevice->ClearDepthStencilView( m_pHDRDepthDSV, D3D10_CLEAR_DEPTH, 1.0, 0 );
        pd3dDevice->OMSetRenderTargets(1,&m_pHDRRTRTV,m_pHDRDepthDSV);
    }

    m_skybox.OnFrameRender(mView*mProj,0.7f,D3DXVECTOR4(0.7f,0.5f,0.08f,0));
    //m_skybox.OnFrameRender(mView*mProj);

    pd3dDevice->IASetPrimitiveTopology(D3D10_PRIMITIVE_TOPOLOGY_TRIANGLELIST);

    switch(m_iInstancingMode)
    {
        /// Single Draws
    case 0:
        V(DrawSorted(pd3dDevice,mView,mProj,time));
        break;
        // Hybrid instancing
    case 1:
    default:
        V(DrawHybridInstanced(pd3dDevice,mView,mProj,time));
        break;
    }

    // Now Draw our ground plane, this is simple DX10 std rendering
    D3DXMATRIX mWorld;
    D3DXMATRIX mScale;
    D3DXMATRIX mTranslation;
    D3DXMatrixIdentity(&mWorld);
    D3DXMatrixScaling(&mScale,1.f,1.f,1.f);   //$$ These tweaks should be mograted to source art....s
    D3DXMatrixTranslation(&mTranslation,0,4.5f,0);
    D3DXMATRIX mWorldViewProj = mScale * mTranslation * mView * mProj;

    m_pWorldVar->SetMatrix((float*)&mWorld);
    m_pWorldViewProjectionVar->AsMatrix()->SetMatrix((float*)&mWorldViewProj);
    m_pNormalTexVar->SetResource(TextureLibrary::singleton()->GetShaderResourceView("normalmap"));
    pd3dDevice->IASetInputLayout(m_pInputLayoutTerrain);
    pd3dDevice->IASetIndexBuffer(NULL,DXGI_FORMAT_R32_UINT,0);
    m_GroundMesh.Render(pd3dDevice,m_pTechTerrain,m_pDiffuseTexVar,0,0,(DWORD)-1,m_pNormalTexVar);
    if(m_GroundMesh.m_pMesh10)
    {
        m_polysDrawnLastFrame += m_GroundMesh.m_pMesh10->GetFaceCount();
        m_drawCallsLastFrame += 1;
    }
    
    // A little blur and blend and then a gamma correction pass
    if(m_bHDR)
    {
        UINT uiVertexStride = 0;
        UINT uOffset = 0;
        ID3D10Buffer *vertBuffer = NULL;
        ID3D10EffectPass *pPass = NULL;

        // Setup FS effects - just a simple blur-glow effect
        pd3dDevice->IASetIndexBuffer(NULL,DXGI_FORMAT_R32_UINT,0);
        pd3dDevice->IASetVertexBuffers(0,1,&vertBuffer, &uiVertexStride, &uOffset );
        pd3dDevice->IASetInputLayout(NULL);
        pd3dDevice->IASetPrimitiveTopology(D3D10_PRIMITIVE_TOPOLOGY_TRIANGLESTRIP);

        // No do blur passes H 
        pd3dDevice->ClearRenderTargetView( m_pHDRRT2RTV, ClearColor );
        pd3dDevice->OMSetRenderTargets(1,&m_pHDRRT2RTV,m_pHDRDepthDSV);
        m_pEffect10->GetVariableBySemantic("HDR")->AsShaderResource()->SetResource(m_pHDRRTSRV);
        pPass = m_pEffect10->GetTechniqueByName("BlurH")->GetPassByIndex(0);
        pPass->Apply(0);
        pd3dDevice->Draw(4,0);  // 4 verts for a FS quad

        // then V
        pd3dDevice->ClearRenderTargetView( m_pHDRRT3RTV, ClearColor );
        pd3dDevice->OMSetRenderTargets(1,&m_pHDRRT3RTV,m_pHDRDepthDSV);
        m_pEffect10->GetVariableBySemantic("HDR")->AsShaderResource()->SetResource(m_pHDRRT2SRV);
        pPass = m_pEffect10->GetTechniqueByName("BlurV")->GetPassByIndex(0);
        pPass->Apply(0);
        pd3dDevice->Draw(4,0);  // 4 verts for a FS quad

        // reset to BB
        pd3dDevice->OMSetRenderTargets(1,&pOldRTV,pOldDSV);

        // $ Skipping luminance calculation in favor of manual gamma and exposure in GUI.

        //D3DX10SaveTextureToFile(m_pHDRRT,D3DX10_IFF_DDS,L"HDROutput.dds");

        m_pEffect10->GetVariableBySemantic("HDR")->AsShaderResource()->SetResource(m_pHDRRTSRV);
        m_pEffect10->GetVariableBySemantic("HDRBLUR")->AsShaderResource()->SetResource(m_pHDRRT3SRV);
        pPass = m_pTechHDR->GetPassByIndex(0);
        pPass->Apply(0);
        pd3dDevice->Draw(4,0);  // 4 verts for a FS quad
    }

    return S_OK;    
}

/*
    Very dumb Non Instanced Draw.  This just iterates over the instances, and updates VS constants for each animation, 
    then draws each character given it's world transform.  
    
    $$ This method is NO LONGER USED as the smarter DrawSorted method is faster.  this is just included for reference.
*/
HRESULT ArmyManager::Draw(ID3D10Device *pd3dDevice,D3DXMATRIX &mView,D3DXMATRIX &mProj, float time,int maxLOD)
{
    pd3dDevice->IASetInputLayout(m_pInputLayout);

    // iterate over the # of charactes to draw
    int realNumToDraw = min((int)m_Characters.size(),m_NumToDraw);
    for(int currentCharacter = 0;currentCharacter<realNumToDraw;currentCharacter++)
    {
        AnimatedCharacterInstance *pCharacter = m_Characters[currentCharacter];
        if(pCharacter->LOD < 0 || pCharacter->LOD > maxLOD) continue;

        D3DXMATRIX mWorldViewProj = pCharacter->mWorld * mView * mProj;
        m_pWorldViewProjectionVar->AsMatrix()->SetMatrix((float*)&mWorldViewProj);
        m_pWorldVar->SetMatrix((float*)&pCharacter->mWorld);

        // Grab the current frame of animation and set it to constants
        
        ID3D10EffectTechnique *pRenderTechnique = NULL;
        if(pCharacter->LOD == 0 || pCharacter->LOD == 1)
        {
            pRenderTechnique = m_pTechAnimation[0];
        }
        else if(pCharacter->LOD == 2)
        {
            pRenderTechnique = m_pTechAnimation[1];
        }
        else
        {
            pRenderTechnique = m_pTechAnimation[2];
        }

        if(m_CharacterLODs[pCharacter->LOD]->m_animations.size())
        {
            CharacterAnimation *pAnimation = m_CharacterLODs[pCharacter->LOD]->m_animations[pCharacter->animation];
            D3DXMATRIX *frameMatrices = pAnimation->GetEncodedFrameAt(pCharacter->animationTime);
            m_pEffect10->GetVariableByName("g_matrices")->AsMatrix()->SetMatrixArray(*frameMatrices,0,m_CharacterLODs[pCharacter->LOD]->numBones);
        }

        m_pEffect10->GetVariableByName("g_instanceColor")->AsVector()->SetFloatVector((float*)&pCharacter->color);

        AnimatedCharacter *referenceCharacter = m_CharacterLODs[pCharacter->LOD];

        // draw each submesh on each character, optionally skipping if we dont want to draw.
        std::string previousTexture = "null";
        for(int iSubMesh=0;iSubMesh<(int)referenceCharacter->m_characterMeshes.size();iSubMesh++)
        {
            // m_characterMeshes are the list of submeshes for the character.
            AnimatedCharacter::SubMesh &subMesh = referenceCharacter->m_characterMeshes[iSubMesh];

            // skip sub meshes if they don't match our submesh set. $$ Note this logic should match GS logic.
            if(subMesh.meshSet != 0 && !(subMesh.meshSet & pCharacter->meshSet)) continue;

            // Set our texture if we need to.
            if(previousTexture.compare(subMesh.texture) != 0)
            {
                m_pDiffuseTexVar->SetResource(TextureLibrary::singleton()->GetShaderResourceView(subMesh.texture));
                m_pNormalTexVar->SetResource(TextureLibrary::singleton()->GetShaderResourceView(subMesh.normal));
                previousTexture = subMesh.texture;
            }

            // Ensure no second buffer left over from instancing...
            UINT uiVertexStride = sizeof(AnimatedCharacter::Vertex);
            UINT uOffset = 0;

            ID3D10Buffer *vertBuffer = subMesh.pVB;
            ID3D10Buffer *indexBuffer = subMesh.pIB;

            pd3dDevice->IASetVertexBuffers(0,1,&vertBuffer, &uiVertexStride, &uOffset );
            if(indexBuffer)
                pd3dDevice->IASetIndexBuffer(indexBuffer,DXGI_FORMAT_R32_UINT,0);

            // Apply the technique contained in the effect
            D3D10_TECHNIQUE_DESC techDesc;
            memset( &techDesc, 0, sizeof( D3D10_TECHNIQUE_DESC ) );
            pRenderTechnique->GetDesc( &techDesc );
            for (UINT iPass = 0; iPass < techDesc.Passes; iPass++)
            {
                ID3D10EffectPass *pCurrentPass = pRenderTechnique->GetPassByIndex( iPass );
                pCurrentPass->Apply( 0 );

                if(indexBuffer)
                    pd3dDevice->DrawIndexed(subMesh.numIndices,0,0);
                else
                    pd3dDevice->Draw(subMesh.numVertices,0);

                m_polysDrawnLastFrame += subMesh.numIndices/3;
                m_drawCallsLastFrame += 1;
            }
        }
    }
    return S_OK;
}

/*
    This piggy backs on the setup of the instanced draws to allow sorted single draw calls.  Grouped by mesh.  Faster than the dumb iterate over characters and draw method above.
*/
HRESULT ArmyManager::DrawSorted(ID3D10Device *pd3dDevice,D3DXMATRIX &mView,D3DXMATRIX &mProj, float time,int maxLOD)
{
    pd3dDevice->IASetInputLayout(m_pInputLayout);

    std::string previousTexture = "null";
    ID3D10Buffer *vertBuffer = NULL;
    for(int iDrawState=0;iDrawState<(int)m_InstancedDraws.size();iDrawState++)
    {
        InstancedDrawCallState &drawState = m_InstancedDraws[iDrawState];

        if(drawState.numToDraw == 0 || drawState.lod > maxLOD) continue;

        int iMesh = m_InstancedDraws[iDrawState].meshIndex; // straight mapping
        AnimatedCharacter::SubMesh &subMesh = drawState.referenceCharacter->m_characterMeshes[iMesh];

        // Ensure no second buffer left over from instancing...
        UINT uiVertexStride = sizeof(AnimatedCharacter::Vertex);
        UINT uOffset = 0;

        // Don't redundant'y set the mesh if we don't need to
        ID3D10Buffer *indexBuffer = subMesh.pIB;
        if(subMesh.pVB != vertBuffer)
        {
            vertBuffer = subMesh.pVB;
        
            pd3dDevice->IASetVertexBuffers(0,1,&vertBuffer, &uiVertexStride, &uOffset );
            if(indexBuffer)
                pd3dDevice->IASetIndexBuffer(indexBuffer,DXGI_FORMAT_R32_UINT,0);
        }

        // Set our texture if we need to.
        if(previousTexture.compare(subMesh.texture) != 0)
        {
            m_pDiffuseTexVar->SetResource(TextureLibrary::singleton()->GetShaderResourceView(subMesh.texture));
            m_pNormalTexVar->SetResource(TextureLibrary::singleton()->GetShaderResourceView(subMesh.normal));
            previousTexture = subMesh.texture;
        }

        ID3D10EffectTechnique *pRenderTechnique = NULL;
        if(drawState.lod == 0 || drawState.lod == 1)
        {
            pRenderTechnique = m_pTechAnimation[0];
        }
        else if(drawState.lod == 2)
        {
            pRenderTechnique = m_pTechAnimation[1];
        }
        else
        {
            pRenderTechnique = m_pTechAnimation[2];
        }

        AnimatedCharacter *referenceCharacter = drawState.referenceCharacter;

        for(int iDraw=0;iDraw<drawState.numToDraw;iDraw++)
        {
            AnimatedCharacterInstance *pCharacter = m_Characters[drawState.instancesUsingThisMesh[iDraw]];

            if(referenceCharacter->m_animations.size())
            {
                CharacterAnimation *pAnimation = referenceCharacter->m_animations[pCharacter->animation];
                D3DXMATRIX *frameMatrices = pAnimation->GetEncodedFrameAt(pCharacter->animationTime);
                m_pEffect10->GetVariableByName("g_matrices")->AsMatrix()->SetMatrixArray(*frameMatrices,0,referenceCharacter->numBones);
            }

            m_pEffect10->GetVariableByName("g_instanceColor")->AsVector()->SetFloatVector((float*)&pCharacter->color);

            D3DXMATRIX mWorldViewProj = pCharacter->mWorld * mView * mProj;
            m_pWorldViewProjectionVar->AsMatrix()->SetMatrix((float*)&mWorldViewProj);
            m_pWorldVar->SetMatrix((float*)&pCharacter->mWorld);

            // Apply the technique contained in the effect
            D3D10_TECHNIQUE_DESC techDesc;
            memset( &techDesc, 0, sizeof( D3D10_TECHNIQUE_DESC ) );
            pRenderTechnique->GetDesc( &techDesc );
            for (UINT iPass = 0; iPass < techDesc.Passes; iPass++)
            {
                ID3D10EffectPass *pCurrentPass = pRenderTechnique->GetPassByIndex( iPass );
                pCurrentPass->Apply( 0 );

                if(indexBuffer)
                    pd3dDevice->DrawIndexed(subMesh.numIndices,0,0);
                else
                    pd3dDevice->Draw(subMesh.numVertices,0);

                m_polysDrawnLastFrame += subMesh.numIndices/3;
                m_drawCallsLastFrame += 1;
            }
        }
    }
    return S_OK;
}

/*
    Instanced Draw.  Binds the instance data buffer, and mesh buffer.  Also binds animation texture and color texture arrays
*/
HRESULT ArmyManager::DrawHybridInstanced(ID3D10Device *pd3dDevice,D3DXMATRIX &mView,D3DXMATRIX &mProj, float time,int maxLOD)
{
    HRESULT hr;

    // Common to all isntanced draws
    D3DXMATRIX mWorldViewProj = mView * mProj;
    m_pWorldViewProjectionVar->AsMatrix()->SetMatrix((float*)&mWorldViewProj);

    pd3dDevice->IASetInputLayout(m_pInputLayout);

    // One instanced draw per submesh
    std::string previousTexture = "null";
    ID3D10Buffer *vertBuffer = NULL;
    ID3D10ShaderResourceView *pAnimationSRV = NULL;
    for(int iDrawState=0;iDrawState<(int)m_InstancedDraws.size();iDrawState++)
    {
        InstancedDrawCallState &drawState = m_InstancedDraws[iDrawState];

        if(drawState.numToDraw == 0 || drawState.lod > maxLOD) continue;

        int iMesh = m_InstancedDraws[iDrawState].meshIndex; // straight mapping
        AnimatedCharacter::SubMesh &subMesh = m_InstancedDraws[iDrawState].referenceCharacter->m_characterMeshes[iMesh];

        if(pAnimationSRV != m_pAnimationResourceView[drawState.lod])
        {
            pAnimationSRV = m_pAnimationResourceView[drawState.lod];
            V(m_pAnimationsVar->SetResource(pAnimationSRV));
        
            D3D10_TEXTURE2D_DESC desc;
            m_pAnimationTexture[drawState.lod]->GetDesc(&desc);
            m_pEffect10->GetVariableByName("g_InstanceMatricesWidth")->AsScalar()->SetInt(desc.Width);
        }

        // Set our texture if we need to.
        if(previousTexture.compare(subMesh.texture) != 0)
        {
            m_pDiffuseTexVar->SetResource(TextureLibrary::singleton()->GetShaderResourceView(subMesh.texture));
            m_pNormalTexVar->SetResource(TextureLibrary::singleton()->GetShaderResourceView(subMesh.normal));
            previousTexture = subMesh.texture;
        }

        UINT uiVertexStride = sizeof(AnimatedCharacter::Vertex);
        UINT uOffset = 0;

        // Don't redundant'y set the mesh if we don't need to
        ID3D10Buffer *indexBuffer = subMesh.pIB;
        if(subMesh.pVB != vertBuffer)
        {
            vertBuffer = subMesh.pVB;
        
            pd3dDevice->IASetVertexBuffers(0,1,&vertBuffer, &uiVertexStride, &uOffset );
            if(indexBuffer)
                pd3dDevice->IASetIndexBuffer(indexBuffer,DXGI_FORMAT_R32_UINT,0);
        }

        // Now loop over internal instance data constant buffers, since we are limited in the # we can fit in a single
        //   constant buffer
        unsigned int buffersUsed = (drawState.numToDraw / INTERNAL_MAX_INSTANCES_PER_BUFFER)+1;
        for(unsigned int buffer = 0;buffer < buffersUsed ; buffer++)
        {
            int numInstances = INTERNAL_MAX_INSTANCES_PER_BUFFER;
            if(buffer == buffersUsed - 1)       // restrict drawing all instances in the final buffer to get an exact count
                numInstances = drawState.numToDraw % INTERNAL_MAX_INSTANCES_PER_BUFFER;

            ID3D10Buffer *pEffectConstantBuffer = NULL;
            m_pEffect10->GetConstantBufferByName("cInstanceData")->AsConstantBuffer()->GetConstantBuffer(&pEffectConstantBuffer);

            // Copy ourselves into the subregion
            D3D10_BUFFER_DESC srcDesc;
            D3D10_BUFFER_DESC dstDesc;
            drawState.pInstanceDataBuffer[buffer]->GetDesc(&srcDesc);
            pEffectConstantBuffer->GetDesc(&dstDesc);

            D3D10_BOX updateBox;
            ZeroMemory(&updateBox,sizeof(D3D10_BOX));
            updateBox.left = 0;
            updateBox.right = updateBox.left + numInstances*sizeof(InstanceDataElement);
            updateBox.top = 0;
            updateBox.bottom = 1;
            updateBox.front = 0;
            updateBox.back = 1;

            pd3dDevice->CopySubresourceRegion(pEffectConstantBuffer,0,0,0,0,drawState.pInstanceDataBuffer[buffer],0,&updateBox);
            SAFE_RELEASE(pEffectConstantBuffer);

            // Apply the technique contained in the effect, and then draw!
            D3D10_TECHNIQUE_DESC techDesc;
            memset( &techDesc, 0, sizeof( D3D10_TECHNIQUE_DESC ) );
            ID3D10EffectTechnique *pTechnique = NULL;
            if(drawState.lod == 0 || drawState.lod == 1)
            {
                pTechnique = m_pTechAnimationInstanced[0];
            }
            else if(drawState.lod == 2)
            {
                pTechnique = m_pTechAnimationInstanced[1];
            }
            else
            {
                pTechnique = m_pTechAnimationInstanced[2];
            }
            pTechnique->GetDesc( &techDesc );
            for (UINT iPass = 0; iPass < techDesc.Passes; iPass++)
            {
                ID3D10EffectPass *pCurrentPass = pTechnique->GetPassByIndex( iPass );
                pCurrentPass->Apply( 0 );

                if(indexBuffer)
                    pd3dDevice->DrawIndexedInstanced(subMesh.numIndices,numInstances,0,0,0);
                else
                    pd3dDevice->DrawInstanced(subMesh.numVertices,numInstances,0,0);

                m_polysDrawnLastFrame += numInstances * subMesh.numIndices/3;
                m_drawCallsLastFrame += 1;
            }
        }   // buffers loop
    }   // instance draw state loop

    return S_OK;
}



/*
    This creates the actual DX10 buffer object that will hold our per instance data.
*/
HRESULT ArmyManager::CreateInstanceBuffer( ID3D10Device* pd3dDevice, ID3D10Buffer **pBuffer, int maxCharacters )
{
    HRESULT hr = S_OK;

    // Create a resource with the input matrices
    D3D10_BUFFER_DESC bufferDesc =
    {
        maxCharacters * sizeof( InstanceDataElement ),
        D3D10_USAGE_DEFAULT,
        D3D10_BIND_VERTEX_BUFFER|D3D10_BIND_SHADER_RESOURCE,
        0,
        0
    };

    V(pd3dDevice->CreateBuffer( &bufferDesc, NULL, pBuffer ));

    return hr;
}

void ArmyManager::FillDataElement(InstanceDataElement*pData,int character)
{
    AnimatedCharacterInstance* pCharacter = m_Characters[character];
    assert(pCharacter);

    D3DXMATRIX mWorld = pCharacter->mWorld;

    // kind of an encoding.  put in the 3x3 rotation scale, plus the translation at the end
    //  allows my 4x4 matrix to fit into a 3x4
    pData->world1 = D3DXVECTOR4(mWorld._11,mWorld._12,mWorld._13,mWorld._41);
    pData->world2 = D3DXVECTOR4(mWorld._21,mWorld._22,mWorld._23,mWorld._42);
    pData->world3 = D3DXVECTOR4(mWorld._31,mWorld._32,mWorld._33,mWorld._43);

    // this is the real meat, the animation offset and the character mesh set.
    pData->animationIndex = GetAnimationOffset(m_CharacterLODs[pCharacter->LOD],pCharacter->animation);
    pData->attachmentSet = pCharacter->meshSet;

    pData->color.r = pCharacter->color.x;
    pData->color.g = pCharacter->color.y;
    pData->color.b = pCharacter->color.z;

    // this is offset in elements, so # bones * frame index
    CharacterAnimation *pAnimation = m_CharacterLODs[pCharacter->LOD]->GetAnimation(pCharacter->animation);
    if(pAnimation)
    {
        UINT texelsPerBone = 4;
        pData->frameOffset = texelsPerBone*m_CharacterLODs[pCharacter->LOD]->numBones * pAnimation->GetFrameIndexAt(pCharacter->animationTime);    
        pData->lerpValue = 0;   // $ lerp Value is unimplemented right now
    }
}


/*
    Updates constant buffers for each instanced draw state element.
*/
HRESULT ArmyManager::UpdateInstancedDrawState(ID3D10Device *pd3dDevice, bool bUpdateBuffers)
{
    // Some calcs about size and lod level locations...
    int size = 0;
    int lodLevels[4];
    for(int i=0;i<4;i++)
    {
        size += (int)m_CharacterLODs[i]->m_characterMeshes.size();
        lodLevels[i] = size;
    }

    // initially make the buffers, or remake
    if(m_InstancedDraws.size() == 0)
    {
        // resize to # of meshes
        m_InstancedDraws.resize(size);

        // Now make the instance data buffers based on usage
        for(int iDraw=0;iDraw<(int)m_InstancedDraws.size();iDraw++)
        {
            for(int i=0;i<4;i++)
            {
                if(iDraw < lodLevels[i])
                {
                    m_InstancedDraws[iDraw].referenceCharacter = m_CharacterLODs[i];
                    m_InstancedDraws[iDraw].meshIndex = i==0?iDraw:iDraw-lodLevels[i-1];
                    break;
                }
            }

            InstancedDrawCallState &drawState = m_InstancedDraws[iDraw];
            for(int i=0;i<INTERNAL_MAX_BUFFERS;i++)
                CreateInstanceBuffer(pd3dDevice,&drawState.pInstanceDataBuffer[i],(int)INTERNAL_MAX_INSTANCES_PER_BUFFER);
        }
    }

    // first clear
    for(int iDraw=0;iDraw<(int)m_InstancedDraws.size();iDraw++)
        m_InstancedDraws[iDraw].instancesUsingThisMesh.clear();

    // allocate instances into each draw call
    for(int iInstance=0;iInstance<min(m_NumToDraw,(int)m_Characters.size());iInstance++)
    {
        if(m_Characters[iInstance]->LOD < 0) continue;  // skip characters that don't want to be drawn...

        AnimatedCharacter *referenceCharacter = m_CharacterLODs[m_Characters[iInstance]->LOD];
        int lodOffset = m_Characters[iInstance]->LOD==0?0:lodLevels[m_Characters[iInstance]->LOD-1];

        for(int iMesh=0;iMesh<(int)referenceCharacter->m_characterMeshes.size();iMesh++)
        {
            if(referenceCharacter->m_characterMeshes[iMesh].meshSet == 0 || m_Characters[iInstance]->meshSet & referenceCharacter->m_characterMeshes[iMesh].meshSet)
            {
                m_InstancedDraws[lodOffset + iMesh].instancesUsingThisMesh.push_back(iInstance);
                m_InstancedDraws[lodOffset + iMesh].lod = m_Characters[iInstance]->LOD;
            }
        }
    }

    // for each single mesh, make the state for the instanced rendering...
    HRESULT hr = S_OK;
    for(int iDraw=0;iDraw<(int)m_InstancedDraws.size();iDraw++)
    {
        InstancedDrawCallState &drawState = m_InstancedDraws[iDraw];

        // Re calc # to draw by traversing backwards
        drawState.numToDraw = (int)drawState.instancesUsingThisMesh.size();

        // Only update buffers if we really are instancing so the single draw case isn't slowed down...
        // also don't bopther to update if there aren't any IN this draw call...
        if(bUpdateBuffers && drawState.numToDraw > 0)
        {
            // update our array of constant buffers...
            unsigned int buffersUsed = (drawState.numToDraw / INTERNAL_MAX_INSTANCES_PER_BUFFER)+1;
            for(unsigned int buffer = 0;buffer < buffersUsed ; buffer++)
            {
                int numInstances = INTERNAL_MAX_INSTANCES_PER_BUFFER;
                if(buffer == buffersUsed - 1)       // restrict drawing all instances in the final buffer to get an exact count
                    numInstances = drawState.numToDraw % INTERNAL_MAX_INSTANCES_PER_BUFFER;
            
                if(!drawState.pInstanceDataBuffer) return E_INVALIDARG;

                D3D10_BUFFER_DESC bDesc;
                drawState.pInstanceDataBuffer[buffer]->GetDesc(&bDesc);

                memset((void*)m_pDataTemp,0,bDesc.ByteWidth); // clear from previous runs

                for(int i=0;i<numInstances;i++)
                {
                    // use lookup list to find the index into the full character list
                    FillDataElement(&(m_pDataTemp[i]),drawState.instancesUsingThisMesh[buffer*INTERNAL_MAX_INSTANCES_PER_BUFFER+i]);
                }

                pd3dDevice->UpdateSubresource(drawState.pInstanceDataBuffer[buffer],D3D10CalcSubresource(0,0,1),NULL,(void*)m_pDataTemp,bDesc.ByteWidth,1);
            }
        }
    }
    return hr;
}

/*
    This creates and populates a texture that contains all the animation data for all frames of all m_animations.
*/
HRESULT ArmyManager::CreateAnimationTexture(ID3D10Device* pd3dDevice,AnimatedCharacter *pAnimatedCharacter, ID3D10Texture2D **ppAnimationTexture, ID3D10ShaderResourceView **ppAnimationResourceView)
{
    if(!pAnimatedCharacter->m_animations.size()) return S_OK;

    // Loop through all m_animations and find the total size of all m_animations in bones
    int maxBones=0;
    for(int currentAnimation=0;currentAnimation<(int)pAnimatedCharacter->m_animations.size();currentAnimation++)
    {
        CharacterAnimation *pAnimation = pAnimatedCharacter->m_animations[currentAnimation];
        maxBones += (int)pAnimation->frames.size() * pAnimatedCharacter->numBones;
    }

    UINT texelsPerBone = 4;

    UINT pixelCount = maxBones * texelsPerBone;    // rowsPerBone lines per matrix, since no projection
    UINT texWidth = 0;
    UINT texHeight = 0;

    // This basically fits the animation into a roughly square texture where the 
    //      width is a multiple of rowsPerBone(our size requirement for matrix storage)
    //      AND both dimensions are power of 2 since it seems to fail without this...
    texWidth = (int)sqrt((float)pixelCount)+1;    // gives us a starting point
    texHeight = 1;
    while(texHeight < texWidth) 
        texHeight = texHeight<<1;
    texWidth = texHeight;

    HRESULT hr = S_OK;
    D3D10_TEXTURE2D_DESC desc;
    ZeroMemory( &desc, sizeof(D3D10_TEXTURE2D_DESC) );

    // Create our texture
    desc.MipLevels = 1;
    desc.Usage = D3D10_USAGE_IMMUTABLE;
    desc.BindFlags = D3D10_BIND_SHADER_RESOURCE;
    desc.CPUAccessFlags = 0;
    desc.SampleDesc.Count = 1;
    desc.SampleDesc.Quality = 0;
    desc.ArraySize = 1;
    desc.Width = texWidth;
    desc.Height = texHeight;
    desc.Format = DXGI_FORMAT_R32G32B32A32_FLOAT;

    // Make a offline buffer that is the full size of the texture
    UINT bufferSize = texHeight*texWidth*sizeof(D3DXVECTOR4);
    D3DXVECTOR4 *pData = new D3DXVECTOR4[desc.Width*desc.Height];
    memset((void*)pData,0,bufferSize);    // clear it

    D3DXVECTOR4 * pCurrentDataPtr = pData;
    for(int currentAnimation=0;currentAnimation<(int)pAnimatedCharacter->m_animations.size();currentAnimation++)
    {
        CharacterAnimation *pAnimation = pAnimatedCharacter->m_animations[currentAnimation];

        // now copy in each frame of the animation by copying each bone matrix.
        for(int currentFrame=0;currentFrame<(int)pAnimation->frames.size();currentFrame++)
        {
            // Here we pass through the full matrix, but the 4th row is just padding, weve encoded
            //      all the data into the first 3 rows.
            D3DXMATRIX *pFrameMatrices = (D3DXMATRIX *)pAnimation->GetEncodedFrame(currentFrame);

            for(int iBone=0;iBone < pAnimatedCharacter->numBones;iBone++)
            {
                D3DXVECTOR4 *pFrameMatrixVectors = (D3DXVECTOR4 *)&pFrameMatrices[iBone];
                memcpy((void*)pCurrentDataPtr++,(void*)(pFrameMatrixVectors[0]),sizeof(D3DXVECTOR4));     
                memcpy((void*)pCurrentDataPtr++,(void*)(pFrameMatrixVectors[1]),sizeof(D3DXVECTOR4)); 
                memcpy((void*)pCurrentDataPtr++,(void*)(pFrameMatrixVectors[2]),sizeof(D3DXVECTOR4)); 
                memcpy((void*)pCurrentDataPtr++,(void*)(pFrameMatrixVectors[3]),sizeof(D3DXVECTOR4)); 
            }
        }
    }

    D3D10_SUBRESOURCE_DATA srd;
    srd.pSysMem = (void*)pData;
    srd.SysMemPitch = texWidth*(sizeof(D3DXVECTOR4));
    srd.SysMemSlicePitch = 1;
    V(pd3dDevice->CreateTexture2D( &desc,&srd, ppAnimationTexture));

    delete[] pData;

    // Make a resource view for it
    D3D10_SHADER_RESOURCE_VIEW_DESC SRVDesc;
    ZeroMemory( &SRVDesc, sizeof(SRVDesc) );
    SRVDesc.Format = desc.Format;
    SRVDesc.ViewDimension = D3D10_SRV_DIMENSION_TEXTURE2D;
    SRVDesc.Texture2D.MipLevels = desc.MipLevels;
    V_RETURN(pd3dDevice->CreateShaderResourceView( *ppAnimationTexture, &SRVDesc, ppAnimationResourceView ));

    return hr;
}

/*
    Calculates an offset in texels for the start of an animation
*/
UINT ArmyManager::GetAnimationOffset(AnimatedCharacter *pAnimatedCharacter,int animation)
{
    if(!pAnimatedCharacter->m_animations.size()) return 0;

    int bonesOffset = 0;
    assert(animation < (int)pAnimatedCharacter->m_animations.size());
    for(int currentAnimation=0;currentAnimation<animation;currentAnimation++)
    {
        CharacterAnimation *pAnimation = pAnimatedCharacter->m_animations[currentAnimation];
        bonesOffset += (int)pAnimation->frames.size() * pAnimatedCharacter->numBones;
    }

    return bonesOffset * 4;    // 4 pixels per bone
}

void ArmyManager::Release()
{

    delete [] m_pDataTemp;

    for(int i=0;i<4;i++)
    {
        if(m_CharacterLODs[i]) m_CharacterLODs[i]->Release();
        delete m_CharacterLODs[i];
        SAFE_RELEASE(m_pAnimationResourceView[i]);
        SAFE_RELEASE(m_pAnimationTexture[i]);

    }

    SAFE_RELEASE(m_pInputLayout);
    SAFE_RELEASE(m_pInputLayoutTerrain);

    SAFE_RELEASE(m_pEffect10);

    while(m_Characters.size() > 0)
    {
        AnimatedCharacterInstance *c = m_Characters.back();
        delete c;
        m_Characters.pop_back();
    }

    while(m_InstancedDraws.size() > 0)
    {
        for(int i=0;i<INTERNAL_MAX_BUFFERS;i++)
            SAFE_RELEASE(m_InstancedDraws.back().pInstanceDataBuffer[i]);
        m_InstancedDraws.back().instancesUsingThisMesh.clear();
        m_InstancedDraws.pop_back();
    }

    m_GroundMesh.Destroy();

    SAFE_RELEASE(m_pHDRRT);
    SAFE_RELEASE(m_pHDRRTRTV);
    SAFE_RELEASE(m_pHDRRTSRV);

    SAFE_RELEASE(m_pHDRRT2);
    SAFE_RELEASE(m_pHDRRT2RTV);
    SAFE_RELEASE(m_pHDRRT2SRV);

    SAFE_RELEASE(m_pHDRRT3);
    SAFE_RELEASE(m_pHDRRT3RTV);
    SAFE_RELEASE(m_pHDRRT3SRV);
    
    SAFE_RELEASE(m_pHDRDepth);
    SAFE_RELEASE(m_pHDRDepthDSV);


    SAFE_RELEASE(m_pSMRT);
    SAFE_RELEASE(m_pSMRTRTV);
    SAFE_RELEASE(m_pSMRTSRV);
   

    m_skybox.OnDestroyDevice();
    
}