static float GlossTop = 0.46;
static float GlossBot = 0.41;
static float GlossDrop = 0.25;

float fresnel(float3 eyeVec, float3 normal, float R0)
{
    float kOneMinusEdotN  = 1-abs(dot(eyeVec, normal));    // note: abs() makes 2-sided materials work

    // raise to 5th power
    float result = kOneMinusEdotN * kOneMinusEdotN;
    result = result * result;
    result = result * kOneMinusEdotN;
    result = R0 + (1-R0) * result;

    return result;
}

float glossy_drop(float v,
		    uniform float top,
		    uniform float bot,
		    uniform float drop)
{
    return (drop+smoothstep(bot,top,v)*(1.0-drop));
}

void shared_lighting_calculations(VS_to_PS IN,
		out float3 DiffuseContrib,
		out float3 SpecularContrib,
		out float3 ReflectionContrib)
{
    float3 Ln = normalize(g_lightPos - IN.worldPos);    
    float3 Nn = normalize(IN.norm);
    float3 Vn = normalize(g_EyePos - IN.worldPos.xyz);
    float3 Hn = normalize(Vn + Ln);
    float4 litV = lit(dot(Ln,Nn),dot(Hn,Nn),SpecExpon);
    
    DiffuseContrib = litV.y * g_lightColor + AmbientScale*AmbiColor;
    
    float spec = litV.z * (glossy_drop(litV.z,GlossTop,GlossBot,GlossDrop));
    SpecularContrib = spec * g_lightColor;
    
    float3 reflVect = -reflect(Vn,Nn);
    ReflectionContrib = g_txEnvMap.Sample(g_samLinear,reflVect).xyz;
}